#include "MainFrame.h"
#include "BobDudeApp.h"
#include "xhex.h"
#include <wx/config.h>
#include <wx/msgdlg.h>
#include <wx/icon.h>
#include <wx/filename.h>
#include <wx/file.h>
//#include <wx/artprov.h>
#include <wx/aboutdlg.h>
#include <boost/lexical_cast.hpp>

#include "avrdude.h"
#include "programmer.h"
#include "part.h"
#include "update.h"


//(*InternalHeaders(MainFrame)
#include <wx/settings.h>
#include <wx/intl.h>
#include <wx/string.h>
//*)

//(*IdInit(MainFrame)
const long MainFrame::ID_HTMLWINDOWFILE = wxNewId();
const long MainFrame::ID_HTMLWINDOWPLATFORM = wxNewId();
const long MainFrame::ID_HTMLWINDOWPROGRAM = wxNewId();
const long MainFrame::ID_HTMLWINDOWLOG = wxNewId();
const long MainFrame::ID_GAUGE_PROGRESS = wxNewId();
const long MainFrame::ID_MENUITEM_OPEN = wxNewId();
const long MainFrame::ID_MENUITEM_RESET_MCU = wxNewId();
//*)

wxGauge* progressBar;
std::string infoPrefix;
extern MainFrame * frame;

void doAvrdudeProgress(int percent, double etime) {
  if (frame) {
    progressBar->SetValue(percent);
    std::string text = infoPrefix;
    text += " ";
    text += boost::lexical_cast<std::string>(percent);
    text += "% done";
//  infoText->SetLabel(text.c_str());
//  infoText->Update();
    progressBar->Update();
  }
}


void doAvrdudeMessage(int type, const char * msg) {
  if (frame) {
    if (type>1){
      return;
    }
    wxString text(msg);
    if (text.Trim().IsEmpty()) {
      return;
    }
    if (text.Contains("error")) {
      text = wxString::Format("[%i]: <font color='#ee0000'>%s</font>", type, text);
    } else {
      text = wxString::Format("[%i]: <font color='#666666'>%s</font>", type, text);
    }
    frame->appendLogLine(text);
  }
}

BEGIN_EVENT_TABLE(MainFrame,wxFrame)
  //(*EventTable(MainFrame)
  //*)
END_EVENT_TABLE()

MainFrame::MainFrame(wxWindow* parent,wxWindowID id) {
  xhexfile = 0;
  wxConfig *config = new wxConfig("BobDude", wxEmptyString, wxEmptyString, wxEmptyString, wxCONFIG_USE_LOCAL_FILE);
  wxString strValue = "";
  config->Read("lastFile", &strValue);
  delete config;

  //(*Initialize(MainFrame)
  wxBoxSizer* BoxSizer4;
  wxBoxSizer* BoxSizer2;
  wxBoxSizer* BoxSizer1;

  Create(parent, wxID_ANY, _("BobDude"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE|wxSYSTEM_MENU|wxRESIZE_BORDER|wxCLOSE_BOX, _T("wxID_ANY"));
  SetClientSize(wxSize(444,466));
  SetMinSize(wxSize(450,340));
  SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
  BoxSizer1 = new wxBoxSizer(wxVERTICAL);
  BoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
  HtmlWindowFile = new HTMLButton(this, ID_HTMLWINDOWFILE, wxDefaultPosition, wxSize(211,122), wxHW_SCROLLBAR_NEVER|wxHW_NO_SELECTION, _T("ID_HTMLWINDOWFILE"));
  HtmlWindowFile->SetPage(_("<body bgcolor=\"#039cd6\" text=\"#ffffff\">\n<font size=\"2\">\n<table cellspacing=\"0\"  cellpadding=\"0\">\n<tr><td width=\"60\">File:</td><td><b>test1.xhex</b></td></tr>\n<tr><td>Flash:</td><td>12345 bytes</td></tr>\n<tr><td>EEPROM:</td><td>none</td></tr>\n<tr><td>Fuse-Bits:</td><td>HFUSE, LFUSE</td></tr>\n</table>\n</font>"));
  HtmlWindowFile->SetMinSize(wxSize(-1,122));
  HtmlWindowFile->SetMaxSize(wxSize(-1,122));
  BoxSizer2->Add(HtmlWindowFile, 1, wxALL|wxEXPAND|wxFIXED_MINSIZE|wxALIGN_LEFT|wxALIGN_TOP, 5);
  HtmlWindowPlatform = new HTMLButton(this, ID_HTMLWINDOWPLATFORM, wxDefaultPosition, wxSize(212,122), wxHW_SCROLLBAR_NEVER|wxHW_NO_SELECTION, _T("ID_HTMLWINDOWPLATFORM"));
  HtmlWindowPlatform->SetPage(_("<body bgcolor=\"#f2af13\" text=\"#ffffff\">\n<font size=\"2\">\n<table cellspacing=\"0\"  cellpadding=\"0\">\n<tr><td width=\"80\">Platform:</td><td><b>niboburger-m16-15</b></td></tr>\n<tr><td>MCU:</td><td>ATmega16A</td></tr>\n<tr><td>Bitclock:</td><td>default</td></tr>\n<tr><td>Programmer:</td><td>usbasp</td></tr>\n<tr><td>Interface:</td><td>usb</td></tr>\n<tr><td>Baudrate:</td><td>none</td></tr>\n</table>\n</font>"));
  HtmlWindowPlatform->SetMinSize(wxSize(-1,122));
  HtmlWindowPlatform->SetMaxSize(wxSize(-1,122));
  BoxSizer2->Add(HtmlWindowPlatform, 1, wxALL|wxEXPAND|wxFIXED_MINSIZE|wxALIGN_LEFT|wxALIGN_TOP, 5);
  BoxSizer1->Add(BoxSizer2, 0, wxEXPAND|wxFIXED_MINSIZE|wxALIGN_LEFT|wxALIGN_TOP, 5);
  BoxSizer4 = new wxBoxSizer(wxVERTICAL);
  HtmlWindowProgram = new HTMLButton(this, ID_HTMLWINDOWPROGRAM, wxDefaultPosition, wxSize(433,41), wxHW_SCROLLBAR_NEVER, _T("ID_HTMLWINDOWPROGRAM"));
  HtmlWindowProgram->SetPage(_("<body bgcolor=\"#4b9a91\" text=\"#ffffff\">\n<div align=\"center\"><b>Program!</b></div>"));
  BoxSizer4->Add(HtmlWindowProgram, 0, wxALL|wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP, 5);
  HtmlWindowLog = new wxHtmlWindow(this, ID_HTMLWINDOWLOG, wxDefaultPosition, wxSize(433,146), wxHW_SCROLLBAR_AUTO, _T("ID_HTMLWINDOWLOG"));
  HtmlWindowLog->SetPage(_("<html><body>\n<font size=\"2\">\n"));
  BoxSizer4->Add(HtmlWindowLog, 1, wxALL|wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP, 5);
  BoxSizer1->Add(BoxSizer4, 1, wxALL|wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP, 0);
  GaugeProgress = new wxGauge(this, ID_GAUGE_PROGRESS, 100, wxDefaultPosition, wxSize(374,16), wxGA_SMOOTH|wxNO_BORDER, wxDefaultValidator, _T("ID_GAUGE_PROGRESS"));
  BoxSizer1->Add(GaugeProgress, 0, wxALL|wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP, 5);
  SetSizer(BoxSizer1);
  MenuBar1 = new wxMenuBar();
  MenuFile = new wxMenu();
  MenuItemOpen = new wxMenuItem(MenuFile, ID_MENUITEM_OPEN, _("Open ..."), wxEmptyString, wxITEM_NORMAL);
  MenuFile->Append(MenuItemOpen);
  MenuItemExit = new wxMenuItem(MenuFile, wxID_EXIT, _("Exit"), wxEmptyString, wxITEM_NORMAL);
  MenuFile->Append(MenuItemExit);
  MenuBar1->Append(MenuFile, _("File"));
  Menu1 = new wxMenu();
  MenuItem1 = new wxMenuItem(Menu1, ID_MENUITEM_RESET_MCU, _("Reset MCU"), wxEmptyString, wxITEM_NORMAL);
  Menu1->Append(MenuItem1);
  MenuBar1->Append(Menu1, _("Tools"));
  MenuHelp = new wxMenu();
  MenuItem5 = new wxMenuItem(MenuHelp, wxID_ABOUT, _("About..."), wxEmptyString, wxITEM_NORMAL);
  MenuHelp->Append(MenuItem5);
  MenuBar1->Append(MenuHelp, _("Help"));
  SetMenuBar(MenuBar1);
  FileDialog1 = new wxFileDialog(this, _("Select file"), wxEmptyString, wxEmptyString, _("All supported files|*.bob3;*.xhex;*.hex;*.srec|BOB3 files (*.bob3)|*.bob3|XHex files (*.xhex)|*.xhex|Intel HEX files (*.hex)|*.hex|Motorla S-record file (*.srec)|*.srec"), wxFD_DEFAULT_STYLE, wxDefaultPosition, wxDefaultSize, _T("wxFileDialog"));
  SetSizer(BoxSizer1);
  Layout();
  Center();

  Connect(ID_MENUITEM_OPEN,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&MainFrame::OnMenuItemOpenSelected);
  Connect(wxID_EXIT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&MainFrame::OnMenuItemExitSelected);
  Connect(ID_MENUITEM_RESET_MCU,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&MainFrame::OnMenuItemResetMCU);
  Connect(wxID_ABOUT,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&MainFrame::OnMenuItemAboutSelected);
  //*)

  HtmlWindowFile->setHTML((*(HtmlWindowFile->GetParser()->GetSource())).AfterFirst('\n'));
  HtmlWindowPlatform->setHTML((*(HtmlWindowPlatform->GetParser()->GetSource())).AfterFirst('\n'));
  HtmlWindowProgram->setHTML((*(HtmlWindowProgram->GetParser()->GetSource())).AfterFirst('\n'));

  platformDialog = new PlatformDialog(this);

  //SetBackgroundColour(wxColour(80,80,80));

  //MenuItem5->SetBitmap(wxArtProvider::GetBitmap(wxART_HELP));
  //MenuItemOpen->SetBitmap(wxArtProvider::GetBitmap(wxART_FILE_OPEN));

  FileDialog1->SetPath(strValue);

  SetTitle(PACKAGE_STRING);
#if defined(__WINDOWS__)
  SetIcon(wxIcon(_("aaaa"), wxBITMAP_TYPE_ICO_RESOURCE ));
#endif
  progressBar = GaugeProgress;

/*
  "HKEY_LOCAL_MACHINE\HARDWARE\DEVICEMAP\SERIALCOMM"


  ComboBoxPort->Append(_("COM1"));
  ComboBoxPort->Append(_("COM2"));
  ComboBoxPort->Append(_("COM3"));
  ComboBoxPort->Append(_("COM4"));
  ComboBoxPort->Append(_("COM5"));
  ComboBoxPort->Append(_("COM6"));
  ComboBoxPort->Append(_("COM7"));
  ComboBoxPort->Append(_("COM8"));
*/

  platformDialog->setProgrammers(Programmer::getAvailableProgrammers());
  platformDialog->setParts(Part::getAvailableParts());

  updateGUI();
  appendLogLine("Using configuration " + wxGetApp().conf + "\n");


  /**/
  HtmlWindowFile->setStyleHTML("default", "<body bgcolor=\"#039cd6\" text=\"#ffffff\">");
  HtmlWindowFile->setStyleHTML("hover", "<body bgcolor=\"#05ace6\" text=\"#ffffff\">");
  HtmlWindowFile->setStyleHTML("active", "<body bgcolor=\"#039cd6\" text=\"#cccccc\">");
  HtmlWindowFile->Bind(wxEVT_BUTTON, &MainFrame::OnMenuItemOpenSelected, this);
  //HtmlWindowFile->Bind(wxEVT_BUTTON, &MainFrame::OnMenuItemOpenSelected, this);

  HtmlWindowPlatform->setStyleHTML("default", "<body bgcolor=\"#f2af13\" text=\"#ffffff\">");
  HtmlWindowPlatform->setStyleHTML("hover", "<body bgcolor=\"#ffbf23\" text=\"#ffffff\">");
  HtmlWindowPlatform->setStyleHTML("active", "<body bgcolor=\"#f2af13\" text=\"#cccccc\">");
  HtmlWindowPlatform->Bind(wxEVT_BUTTON, &MainFrame::OnEditPlatform, this);

  HtmlWindowProgram->setStyleHTML("default", "<body bgcolor=\"#4b9a91\" text=\"#ffffff\">");
  HtmlWindowProgram->setStyleHTML("hover", "<body bgcolor=\"#5baaa1\" text=\"#ffffff\">");
  HtmlWindowProgram->setStyleHTML("active", "<body bgcolor=\"#4b9a91\" text=\"#cccccc\">");
  HtmlWindowProgram->setStyleHTML("disabled", "<body bgcolor=\"#4b9a91\" text=\"#aaaaaa\">");
  HtmlWindowProgram->Bind(wxEVT_BUTTON, &MainFrame::OnButtonProgramClick, this);

  platformDialog->initPlatform();
  HtmlWindowPlatform->setHTML(platformDialog->getHTML());
  HtmlWindowPlatform->activateStyle("default");

  HtmlWindowFile->setHTML(getFileHTML());
  HtmlWindowFile->activateStyle("default");

  for (unsigned int i=0; i<wxGetApp().files.size(); i++) {
    loadFile(wxGetApp().files[i]);
  }
}


MainFrame::~MainFrame() {
  delete xhexfile;
  //(*Destroy(MainFrame)
  //*)
}

wxString MainFrame::getFileHTML() {
  if (m_filename=="") {
    return "<div align='center'><b>no file loaded!</b></div>";
  } else {
    wxString res = "<font size=\"2\"><table cellspacing=\"0\"  cellpadding=\"0\">";
    res += "<tr><td width=\"60\">File:</td><td><b>"+m_filename+"</b></td></tr>";
    if ((xhexfile) && (xhexfile->devicePtr)) {
      segment * s = xhexfile->devicePtr->getSegment("flash");
      if (s) {
        res += wxString::Format("<tr><td>Flash:</td><td>%i bytes</td></tr>", s->byteSize);
      } else {
        res += "<tr><td>Flash:</td><td><em>none</em></td></tr>";
      }
      s = xhexfile->devicePtr->getSegment("eeprom");
      if (s) {
        res += wxString::Format("<tr><td>EEPROM:</td><td>%i bytes</td></tr>", s->byteSize);
      } else {
        res += "<tr><td>EEPROM:</td><td><em>none</em></td></tr>";
      }
      wxString fuses = "";
      if (xhexfile->devicePtr->getSegment("efuse")) {
        fuses += "EFUSE";
      }
      if (xhexfile->devicePtr->getSegment("hfuse")) {
        fuses += fuses.empty()?"HFUSE":", HFUSE";
      }
      if (xhexfile->devicePtr->getSegment("lfuse")) {
        fuses += fuses.empty()?"LFUSE":", LFUSE";
      }
      if (fuses.empty()) {
        fuses = "<em>none</em>";
      }
      res += "<tr><td>Fuse-Bits:</td><td>"+fuses+"</td></tr>";
    }

    res += "</table></font>";
    return res;
  }
}

void MainFrame::OnEditPlatform(wxCommandEvent& event) {
    if ( platformDialog->ShowModal() == wxID_OK ) {
      HtmlWindowPlatform->setHTML(platformDialog->getHTML());
    }
}

void MainFrame::OnButtonProgramClick(wxCommandEvent& event) {
  bool success=false;
  clearLog();
  appendLogLine("Programming file <b>"+m_filename+"</b>");
  wxString programmerType = platformDialog->getProgrammer();
  platformDialog->updateSerialPort(true);
  wxString programmerPort = platformDialog->getInterface();
  wxString programmerBaudrate = platformDialog->getBaudrate();
  wxString programmerBitclock = platformDialog->getBitclock();
  wxString devicePart = platformDialog->getMCU().Lower();

  Programmer * prg =  wxGetApp().avrdude->locateProgrammer(static_cast<const char*>(programmerType));
  if (prg==0) {
    wxMessageBox(_("Programmer "+programmerType+" not supported"), _("Error"));
  }

  Part * part = wxGetApp().avrdude->locatePart(static_cast<const char*>(devicePart));
  if (part==0) {
    wxMessageBox(_("Part "+devicePart+" not supported"), _("Error"));
  }

  if (prg && part && xhexfile) {
    bool canceled = false;
    Part * writePart = part->clone();
    Part * verifyPart = part->clone();
    prg->initpgm();
    prg->setup();

    if ((programmerBaudrate!="") && (programmerBaudrate!="default") && (programmerBaudrate!="none")) {
      try {
        unsigned int baudrate = boost::lexical_cast<unsigned int>(programmerBaudrate);
        if (baudrate>0) {
          prg->setBaudrate(baudrate);
        }
      } catch (...) {
        wxMessageBox(_("Baudrate must be a numeric value!"), _("Error"));
        delete writePart;
        delete verifyPart;
        delete part;
        delete prg;
        return;
      }
    }

    if ((programmerBitclock!="") && (programmerBitclock!="default")) {
      try {
        unsigned int bitclock = boost::lexical_cast<unsigned int>(programmerBitclock);
        if (bitclock>0) {
          //prg->setBaudrate(bitclock);
        }
      } catch (...) {
        wxMessageBox(_("Bitclock must be a numeric value!"), _("Error"));
        delete writePart;
        delete verifyPart;
        delete part;
        delete prg;
        return;
      }
    }

    if (prg->open(static_cast<const char*>(programmerPort))) {
      prg->enable();
      prg->initialize(*writePart);

      std::string psig = writePart->get_signature();
      std::string sig = prg->read_signature(*writePart);
      success=true;

      appendLogLine("Connected device signature is 0x"+sig+"\n");

      if ((sig=="000000")||(sig=="ffffff")) {
        appendLogError("No microcontroller detected!");
        wxString msg = "No microcontroller detected\n\nIgnore and continue programming?";
        int res=wxMessageBox(msg, _("Error"),wxYES_NO+wxICON_EXCLAMATION);
        if (res==wxNO) {
          canceled = true;
        } else {
          appendLogLine("<b>Ignoring problem...</b>");
          success = false;
        }
      } else if (psig!=sig) {
        appendLogError("Device signature does not match "+devicePart+" signature (0x"+psig+")\n");
        wxString msg = "The connected microcontroller is not the selected one!\n\nconnected device signature: 0x"+sig+"\n"+devicePart+" signature: 0x"+psig+"\n\nContinue programming?";
        int res=wxMessageBox(msg, _("Error"),wxYES_NO+wxICON_EXCLAMATION);
        if (res==wxNO) {
          canceled = true;
        } else {
          appendLogLine("<b>Ignoring problem...</b>");
          success = false;
        }
      }

      if (canceled) {
        appendLogError("<b>Programming aborted!</b>");
        success = false;
      } else {
        appendLogLine("<b>Erasing chip</b>");
        prg->avr_chip_erase(*writePart);


        if (xhexfile) {
          if (xhexfile->devicePtr) {
            for (unsigned int i=0; i<xhexfile->devicePtr->segmentVec.size(); i++) {
              segment & seg (xhexfile->devicePtr->segmentVec[i]);
              int file_format = avrdude::FMT_AUTO;
              std::string immediate = "";
              appendLogLine("<b>Writing '"+seg.id+"'</b>");
              wxFile * file = new wxFile();
              wxString filename = wxFileName::CreateTempFileName("rd_", file);
              if (seg.format=="ihex") {
                file->Write(seg.content.data(), seg.content.size());
                file_format = avrdude::FMT_IHEX;
              } else if (seg.format=="srec") {
                file->Write(seg.content.data(), seg.content.size());
                file_format = avrdude::FMT_SREC;
              } else if ((seg.dataVec.size()==1) && (seg.dataVec[0].address=="")) {
                immediate = seg.dataVec[0].content;
                if (immediate.size()>2) {
                  if ((immediate[0]!='0') || (immediate[1]!='x')) {
                    immediate = "0x" + immediate;
                  }
                }
                file_format = avrdude::FMT_IMM;
              } else {
                // convert data to ihex lines
                for (unsigned int j=0; j<seg.dataVec.size(); j++) {
                  std::string length = "10";
                  std::string address = seg.dataVec[j].address;
                  std::string data = seg.dataVec[j].content;
                  std::string check = "00";
                  file->Write(":"+length+address+"00"+data+check);
                }
                file->Write(":00000001FF\n");
                file_format = avrdude::FMT_IHEX;
              }

              file->Flush();
              file->Close();
              delete file;
              avrdude::Update * writeUpdate;
              if (file_format==avrdude::FMT_IMM) {
                //wxMessageBox(immediate, _("immediate"));
                writeUpdate = new avrdude::Update(avrdude::DEVICE_WRITE, seg.id, file_format, immediate);
              } else {
                wxString temp = filename.fn_str();
                writeUpdate = new avrdude::Update(avrdude::DEVICE_WRITE, seg.id, file_format, static_cast<const char*>(temp));
              }
              //avrdude::Update * writeUpdate = new avrdude::Update(avrdude::DEVICE_WRITE, seg.id, avrdude::FMT_AUTO, "D:\\Sandbox\\sourceforge\\nibobeelib\\trunk\\hex\\program1.hex");

              if (prg->do_update_op(*writePart, *writeUpdate, false, false)) {
                //wxMessageBox(_("PROGRAMMED"), _("Info"));
              } else {
                appendLogError("<b>Error while writing "+seg.id+"</b>");
                wxMessageBox(_("Programming failed!"), _("Error"));
                success = false;
              };
              wxRemoveFile(filename);
            }
            appendLogLine("Segment finished.");
          }

        }
        if (success) {
          appendLogLine("Programming finished <font color='#00aa00'><b>successfull</b></font>.");
        } else {
          appendLogLine("Programming finished.");
        }
      }
      prg->disable();
      prg->close();
    } else {
      if (programmerPort!="") {
        appendLogError("<b>No programmer connected to "+programmerPort+"!</b>");
        wxMessageBox(_("No programmer connected to "+programmerPort+",\nplease connect!"), _("Error"));
      } else {
        appendLogError("<b>No programmer connected!</b>");
        wxMessageBox(_("No programmer connected,\nplease connect!"), _("Error"));
      }
    }
    delete writePart;
    delete verifyPart;
  }
  delete part;
  delete prg;

}


void MainFrame::OnMenuItemOpenSelected(wxCommandEvent& event) {
  if (FileDialog1->ShowModal()==wxID_OK) {
    loadFile(FileDialog1->GetPath());
  }
}

void MainFrame::loadFile(const wxString& name) {
  clearLog();

  std::string defPlatform = static_cast<const char*>(platformDialog->getPlatform());
  //appendLogLine("New file\n");
  appendLogLine("Loading file <b><font color='#000066'>"+name+"</font></b>\n");
  try {
    delete xhexfile;
    xhexfile = 0;
    xhexfile = xhex::create(static_cast<const char*>(name), defPlatform);
    wxFileName filename = wxFileName::FileName(name);
    m_filename = filename.GetFullName();
    appendLogLine("<b>File import finished!</b>");
  } catch (const std::exception & e) {
    wxString m = e.what();
    appendLogError("<b>Error during file import:</b> "+m);
  }
  if (xhexfile) {
    platformDialog->applyXhex(xhexfile);
    HtmlWindowPlatform->setHTML(platformDialog->getHTML());
  }
  updateGUI();
  HtmlWindowFile->setHTML(getFileHTML());
  updateGUI();

  wxConfig *config = new wxConfig("BobDude", wxEmptyString, wxEmptyString, wxEmptyString, wxCONFIG_USE_LOCAL_FILE);
  wxString strValue = "";
  config->Write("lastFile", name);
  delete config;
}

void MainFrame::appendLogLine(const wxString& text) {
  int x;
  int y;
  HtmlWindowLog->GetViewStart(&x,&y);
  HtmlWindowLog->AppendToPage("<div>"+text+"</div>");
  y+=10;
  HtmlWindowLog->Scroll(0, y);
  HtmlWindowLog->Update();
  //TextCtrlLog->AppendText(text);
  //TextCtrlLog->ShowPosition(TextCtrlLog->GetLastPosition());
  //TextCtrlLog->Update();
}


void MainFrame::clearLog() {
  HtmlWindowLog->SetPage("<html><body><font size=\"2\">");
  HtmlWindowLog->Update();
  progressBar->SetValue(0);
  progressBar->Update();
}


void MainFrame::appendLogError(const wxString& text) {
    appendLogLine("<font color='#ee0000'>"+text+"</font>");
}


void MainFrame::appendLogSuccess(const wxString& text) {
    appendLogLine("<font color='#11dd11'>"+text+"</font>");
}


void MainFrame::OnMenuItemAboutSelected(wxCommandEvent& event) {
  wxAboutDialogInfo info;
  info.SetName(_(PACKAGE_NAME));
  info.SetVersion(_(PACKAGE_VERSION));
  info.SetDescription(_("Use BobDude to transfer BOB3 files to Atmel AVR based B-O-B-3 devices."));
  info.SetCopyright(wxT("(C) 2016 Nils Springob <springob@progbob.org>"));
  info.SetWebSite("http://www.progbob.org");
  wxAboutBox(info);
}


void MainFrame::OnMenuItemExitSelected(wxCommandEvent& event) {
   Close(true);
}

void MainFrame::updateGUI() {
  if (xhexfile==0) {
    HtmlWindowProgram->activateStyle("disabled");
  } else {
    HtmlWindowProgram->activateStyle("default");
  }
}




void MainFrame::OnMenuItemResetMCU(wxCommandEvent& event) {
  bool success=false;
  clearLog();
  appendLogLine("Resetting MCU");
  wxString programmerType = platformDialog->getProgrammer();
  wxString programmerPort = platformDialog->getInterface();
  wxString programmerBaudrate = platformDialog->getBaudrate();
  wxString programmerBitclock = platformDialog->getBitclock();
  wxString devicePart = platformDialog->getMCU().Lower();

  Programmer * prg =  wxGetApp().avrdude->locateProgrammer(static_cast<const char*>(programmerType));
  if (prg==0) {
    wxMessageBox(_("Programmer "+programmerType+" not supported"), _("Error"));
  }

  Part * part = wxGetApp().avrdude->locatePart(static_cast<const char*>(devicePart));
  if (part==0) {
    wxMessageBox(_("Part "+devicePart+" not supported"), _("Error"));
  }

  if (prg && part) {
    bool canceled = false;
    Part * writePart = part->clone();
    Part * verifyPart = part->clone();
    prg->initpgm();
    prg->setup();

    if ((programmerBaudrate!="") && (programmerBaudrate!="default") && (programmerBaudrate!="none")) {
      try {
        unsigned int baudrate = boost::lexical_cast<unsigned int>(programmerBaudrate);
        if (baudrate>0) {
          prg->setBaudrate(baudrate);
        }
      } catch (...) {
        wxMessageBox(_("Baudrate must be a numeric value!"), _("Error"));
        delete writePart;
        delete verifyPart;
        delete part;
        delete prg;
        return;
      }
    }

    if ((programmerBitclock!="") && (programmerBitclock!="default")) {
      try {
        unsigned int bitclock = boost::lexical_cast<unsigned int>(programmerBitclock);
        if (bitclock>0) {
          //prg->setBaudrate(bitclock);
        }
      } catch (...) {
        wxMessageBox(_("Bitclock must be a numeric value!"), _("Error"));
        delete writePart;
        delete verifyPart;
        delete part;
        delete prg;
        return;
      }
    }

    if (prg->open(static_cast<const char*>(programmerPort))) {
      prg->enable();
      prg->initialize(*writePart);

      std::string psig = writePart->get_signature();
      std::string sig = prg->read_signature(*writePart);
      success=true;

      appendLogLine("Connected device signature is 0x"+sig+"\n");

      if ((sig=="000000")||(sig=="ffffff")) {
        appendLogError("No microcontroller detected!");
        canceled=true;
      } else if (psig!=sig) {
        appendLogError("Device signature does not match "+devicePart+" signature (0x"+psig+")\n");
        canceled=true;
      }

      if (canceled) {
        //appendLogError("<b>Programming aborted!</b>");
        success = false;
      } else {
        appendLogSuccess("<b>Reset successfull!</b>");
      }
      prg->disable();
      prg->close();
    } else {
      if (programmerPort!="") {
        appendLogError("<b>No programmer connected to "+programmerPort+"!</b>");
        wxMessageBox(_("No programmer connected to "+programmerPort+",\nplease connect!"), _("Error"));
      } else {
        appendLogError("<b>No programmer connected!</b>");
        wxMessageBox(_("No programmer connected,\nplease connect!"), _("Error"));
      }
    }
    delete writePart;
    delete verifyPart;
  }
  delete part;
  delete prg;
}
